import ptcFwGridBase from './ptc-fw-grid-base.js'
import ptcFwCols from './ptc-fw-cols.js'
import ptcFwGroups from './ptc-fw-groups.js'

export default {
    extends: ptcFwGridBase,
    name: 'ptc-fw-grid-tiles',
    props: [],
    components: {
        'ptc-fw-cols' : ptcFwCols,
        'ptc-fw-groups': ptcFwGroups
    },
    data: function () {
        return {
            newLayout: true,
            gridLayout: {
                groups: []
            }
        }
    },

    mounted() {
        this.createLayout();
    },

    updated() {
        // Create the observer after the DOM has been updated.
        if (this.newLayout) {
            this.$nextTick(() => {
                this.createObserver(`#fw-tiles-${this.group.id}`);
            });
            this.newLayout = false;
        }
    },

    computed: {
        imageHeight() {
            let container = $(`#fw-tiles-${this.group.id}`);
            return ((container.width() / this.layout.elements)-(this.layout.elements*5)) + 'px';
        }
    },

    watch: {
        'layout.elements' : function(newVal,oldVal) {
            this.createLayout();
        },
        'layout.squareThumbs' : function(newVal,oldVal) {
            this.createLayout();
        },
        'group' : function(newVal,oldVal) {
            this.createLayout();
        }
    },

    methods: {
        // Transform the fileData in the layout we need.
        createLayout()
        {
            let self = this;

            function createRows(group,groupFiles)
            {
                group.rows = [];
                // We arrange the files into rows, where each row has layout.elements files.
                let rindex = 0;
                let row = {
                    files: []
                }
                for (let f of groupFiles) {
                    if ((rindex > 0) && (rindex % self.layout.elements) == 0) {
                        // Deep copy!
                        group.rows.push(row);
                        row = {
                            files: []
                        }
                    }
                    row.files.push(f);
                    rindex++;
                }
                if (row.files.length) {

                    // If the last row has less than self.layout.elements
                    // entries we add some dummies?
                    let remain = row.files.length % self.layout.elements;
                    if (remain) {
                        for (let i = 0; i < (self.layout.elements-remain); ++i) {
                            row.files.push({});
                        }
                    }

                    group.rows.push(row);
                }
            }

            function addGroup(parent,group) {
                let g = {
                    id: group.id,
                    name: group.name,
                    level: group.level,
                    type: group.type,
                    groups: [],
                    files: group.files,
                    rows: []
                }
                createRows(g,group.files);

                for (let i of group.groups) {
                    addGroup(g,i);
                }
                parent.groups.push(g);
            }

            this.clearSelection(true);

            this.gridLayout = {
                id: this.group.id,
                name: this.group.name,
                level: 0,
                groups: [],
                files: this.group.files,
                rows: []
            }
            if (this.group.files != undefined) {
                createRows(this.gridLayout,this.group.files);
            }
            for (let g of this.group.groups) {
                addGroup(this.gridLayout,g);
            }

            // console.log(JSON.stringify(this.gridLayout,null,2));

            this.newLayout = true;
        }
    },

    template: `
        <div :id="'fw-tiles-'+group.id" class="fw-tiles-container">

            <ptc-fw-groups v-if="layout.dynamic && group.groups && group.groups.length"
                :group="group"
                :layout="layout"
                @drill-down="drillDown"
            ></ptc-fw-groups>
    
            <div v-if="!layout.dynamic || (group.files && group.files.length)">
                <h3 v-if="group.id != 0">{{group.name}} <span class="float-right">{{group.files ? group.files.length : ''}}</span></h3>
                
                <div v-for="r,index in gridLayout.rows">
                    <div class="card-group fw-tiles-card-group">
                        <div v-for="f,index in r.files" class="card fw-tiles-card" :id="getFileId(f)" :style="{'visibility' : f.id != undefined ? 'unset' : 'hidden'}" :class="getSelectionClass(f.id)" @click="itemClicked(f.id,$event)">
                            <div v-if="f.id != undefined">

                                <div class="card-body fw-tiles-card-header" v-if="getHeader(f)" v-html="getHeader(f)"></div>

                                <div v-if="!layout.squareThumbs" :id="getFileId(f)" class="fw-tiles-image-wrapper" :style="{'max-width': f.imageWidth+'px'}">
                                    <img :data-id="f.id" :src="getPlaceholderUrl(f)" :data-src="getImageUrl(f)" :alt="f.name" class="card-img-top fw-tiles-image ptc-fw-observed-image" :style="{'max-width': f.imageWidth+'px'}">
                                </div>

                                <div v-if="layout.squareThumbs" :data-id="getFileId(f)" :data-src="getImageUrl(f)" class="ptc-fw-observed-image" :style="{'background-image': 'url(' + getPlaceholderUrl(f) + ')','width':'auto', 'height': imageHeight, 'background-repeat': 'no-repeat', 'background-size': 'cover', 'background-position':'center center'}"></div>

                                <ptc-fw-cols
                                    v-if="layout.collectionOptions.enabled"
                                    :file="f"
                                    :options="layout.collectionOptions"
                                ></ptc-fw-cols>
                                    
                                <div class="card-body fw-tiles-card-footer" v-if="getFooter(f)" v-html="getFooter(f)"></div>
                            </div>
                        </div>
                    </div>
                </div>            

                <div v-if="!layout.dynamic">
                    <ptc-fw-grid-tiles
                        v-for="g in gridLayout.groups"
                        :key="g.id"
                        :group="g" 
                        :layout="layout" 
                    ></ptc-fw-grid-tiles>
                </div>
            </div>
        </div>
    `
}


