export default {
    name: 'ptc-groups',
    props: {
        group: {
            type: Object,
            required: true
        },
        layout: {
            type: Object,
            required: true
        }
    },
    components: {
    },
    data: function () {
        return {
            observer: null,
        }
    },

    mounted() {
        this.$nextTick(() => {
            this.createObserver();
        });
    },

    updated() {
    },

    computed: {
        height()
        {
            return Math.min(300,$(window).width() / this.layout.groupOptions.columns);
        },
        
        imageHeight() {
            return this.height + 'px';
        }
    },

    watch: {
        group: function(newVal,oldVal) {
            this.$nextTick(() => {
                this.createObserver();
            });
        }
    },

    methods: {
        getFileId(file) {
            return `g${this.group.id}-c${file.id}`;
        },

        getPlaceholderUrl(file)
        {
            if (file != null) {
                return `data:image/svg+xml,%3Csvg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 ${file.imageWidth} ${file.imageHeight}"%3E%3C/svg%3E`;
            }
            else {
                return `data:image/svg+xml,%3Csvg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 ${this.height} ${this.height}"%3E%3C/svg%3E`;            
            }
        },

        getImageUrl(file)
        {
            if (!file.protected || this.layout.showProtectedFiles) {
                return IMatch.getImageUrl(file.id,this.layout.imageSize);
            }
            else {
                return 'img/fw-shield-icon.svg';
            }
        },

        drillDown(group)
        {
            this.$emit('drill-down',group);
        },

        drillUp()
        {
            this.$emit('drill-up');
        },

        createObserver()
        {
            // See ptc-fw-grid-smart.js for details
            if (this.observer) {
                this.observer.disconnect();
                this.observer = null;
            }

            this.observer = new IntersectionObserver(entries => {
                for (let e of entries) {
                    if (e.isIntersecting) {
                        let img = $(e.target);
                        if (this.layout.groupOptions.squareThumbs) {
                            img.css('background-image', `url(${img.attr('data-src')})`);
                        }
                        else {
                            img.attr('src',img.attr('data-src'));
                        }
                        img.addClass('animated fadeIn');
                        self.observer.unobserve(img[0]);
                    }
                }
            });
            let self = this;
            // We grab all images in the group produced by this component 
            // and add them to the observer.  
            $(`#fw-groups-${this.group.id} .ptc-fw-observed-image`).each(function(e) {
                self.observer.observe(this);
            });
        }
    },

    template: `
        <div :id="'fw-groups-'+group.id" class="fw-groups-container">
            <div class="row">
                <div v-for="g, index in group.groups" class="p-0 m-0" :class="'col-'+layout.groupOptions.columns" :style="{'height': imageHeight}" @click="drillDown(g)">
                    <div class="card fw-groups-card">
                        <div v-if="g.file && (g.file != null)">
                            <div v-if="!layout.groupOptions.squareThumbs" :id="getFileId(g.file)" class="fw-groups-image-wrapper">
                                <img :data-id="g.file.id" :src="getPlaceholderUrl(g.file)" :data-src="getImageUrl(g.file)" class="card-img-top fw-groups-image ptc-fw-observed-image" :style="{'max-width': g.file.imageWidth+'px'}">
                            </div>
                            <div v-if="layout.groupOptions.squareThumbs" :data-id="getFileId(g.file)" :data-src="getImageUrl(g.file)" class="ptc-fw-observed-image fw-groups-image" :style="{'background-image': 'url(' + getPlaceholderUrl(g.file) + ')','width':'100%', 'height': imageHeight, 'background-repeat': 'no-repeat', 'background-size': 'cover', 'background-position':'center center'}"></div>
                        </div>
                        <div v-else>
                            <div class="fw-groups-image ptc-fw-observed-image" :data-src="getPlaceholderUrl(null)" :style="{'background-image': 'url(' + getPlaceholderUrl(null) + ')','width':'100%', 'height': imageHeight, 'background-repeat': 'no-repeat', 'background-size': 'cover', 'background-position':'center center'}"></div>
                        </div>
                        <div v-if="g.id > 0" class="card-img-overlay fw-groups-overlay">
                            <h1>{{g.name}}</h1>
                        </div>
                    </div>
                </div>
            </div>            
        </div>
    `
}


