import ptcFwGridBase from './ptc-fw-grid-base.js'
import ptcFwCols from './ptc-fw-cols.js'

export default {
    extends: ptcFwGridBase,
    name: 'ptc-fw-list',
    props: [],
    components: {
        'ptc-fw-cols' : ptcFwCols
    },
    data: function () {
        return {
            newLayout: true,
            gridLayout: {
                groups: []
            }
        }
    },

    mounted() {
        this.createLayout();
    },

    updated() {
        let self = this;

        if (this.newLayout) {
            this.$nextTick(() => {
                this.createObserver(`#fw-list-${this.group.id}`);
            });
            this.newLayout = false;
        }
    },

    watch: {
        'layout.elements' : function(newVal,oldVal) {
            this.createLayout();
        },
        'layout.squareThumbs' : function(newVal,oldVal) {
            this.createLayout();
        }
    },

    computed: {
        imageSize()
        {
            return (this.layout.elements*20) + 'px';
        },
    },

    methods: {
        // Transform the fileData in the layout we need.
        createLayout()
        {
            this.gridLayout = {}

            function addGroup(parent,group) {
                let g = {
                    name: group.name,
                    id: group.id,
                    level: group.level,
                    type: group.type,
                    files: group.files,
                    groups: []
                }

                for (let i of group.groups) {
                    addGroup(g,i);
                }
                parent.groups.push(g);
            }

            this.gridLayout = {
                id: this.group.id,
                name: this.group.name,
                files: this.group.files,
                groups: []
            }
            for (let g of this.group.groups) {
                addGroup(this.gridLayout,g);
            }

            this.newLayout = true;
        }
    },

    template: `
        <div :id="'fw-list-'+group.id" class="fw-list-container">
            <h3 v-if="gridLayout.name != ''">{{gridLayout.name}}</h3>
            <div class="row m-0 p-0">
                <div v-for="f,index in gridLayout.files" class="card fw-list-card"  :class="[true ? layout.listData.columnClass : '',getSelectionClass(f.id)]" @click.prevent="itemClicked(f.id,$event)">
                    <div class="media fw-list-media">
                        <div v-if="!layout.squareThumbs" class="align-self-start mr-3">
                            <img :id="getFileId(f)" :data-id="f.id" :src="getPlaceholderUrl(f)" :data-src="getImageUrl(f)" :alt="f.name" class="mt-1 ptc-fw-observed-image" :style="{'width' : imageSize, 'height' : 'auto'}"> 
                        </div>
                        <div v-if="layout.squareThumbs" :data-id="getFileId(f)" :data-src="getImageUrl(f)" class="ptc-fw-observed-image mr-3" :style="{'background-image': 'url(' + getPlaceholderUrl(f) + ')','width':imageSize, 'height': imageSize, 'background-repeat': 'no-repeat', 'background-size': 'cover', 'background-position':'center center'}"></div>

                        <div class="media-body fw-list-content">
                            <div v-if="getListContent(f)" v-html="getListContent(f)"></div>
                        </div>
                    </div>

                    <ptc-fw-cols
                        v-if="layout.collectionOptions.enabled"
                        :file="f"
                        :options="layout.collectionOptions"
                    ></ptc-fw-cols>

                </div>
            </div>            

            <ptc-fw-list
                v-for="g in gridLayout.groups"
                :key="g.id"
                :group="g" 
                :layout="layout" 
            ></ptc-fw-list>
        </div>
    `
}


