import ptcFwGridBase from './ptc-fw-grid-base.js'
import ptcFwCols from './ptc-fw-cols.js'


export default {
    extends: ptcFwGridBase,
    name: 'ptc-fw-strip',
    props: [],
    components: {
        'ptc-fw-cols' : ptcFwCols
    },
    data: function () {
        return {
            position: 0,
            vertical: false,
            newLayout: true,
            isPanning: false,
            panOrg: 0,
            panPos: 0
        }
    },

    created() {
    },

    beforeDestroy() {
    },

    updated()
    {
        if (this.newLayout) {
            this.newLayout = false;
            this.$nextTick(() => {
                this.$nextTick(() => {
                    this.createObserver(`#fw-strip-${this.group.id}`);
                });
            });
        }
    },

    mounted() {
        // Each file window can handle only one group. The other layouts
        // do this differently, we just use group[0].
        if (this.group.files == undefined) {
            this.group.files = this.group.groups[0].files;
        }

        // If selection is allowed, we select/focus the first file.
        if (this.layout.allowSelection) {
            if (this.group.files && this.group.files.length) {
                this.select(this.group.files[0].id,true);
                if (this.layout.focusedFile != 0) {
                    this.setFocus(this.layout.focusedFile);
                }
                else {
                    this.setFocus(this.group.files[0].id);
                }
                let fi = this._getFocusIndex();
                if (fi >= 0) {
//                    this.owl.trigger('to.owl',fi,0);
                }
            }
        }

        window.setTimeout(() => {
            this.$forceUpdate();
        },5);
    },

    computed: {
        getHeight()
        {
            return (this.layout.elements*20) + 'px';
        },
        container()
        {
            return $(`#fw-strip-${this.group.id}`);
        }
    },

    watch: {
        'layout.elements': function (newValue,oldValue) {
            this.reset(false);
        },
        'group.files': function (newValue,oldValue) {
            this.reset(true);
        },
        'layout.squareThumbs' : function(oldVal,newVal) {
            this.reset(true);
        }
    },

    methods: {

        reset(clearSSelection)
        {
            if (clearSSelection === true) {
                this.lastClicked = 0;                
                this.clearSelection();
            }

            this.newLayout = true;
        },

        getWidth(file)
        {
            if (this.layout.squareThumbs) {
                return this.getHeight;
            }
            else {
                return Math.ceil(file.imageWidth / (file.imageHeight / (this.layout.elements*20))) + 'px';
            }
        },

        getDynamicSize(file)
        {
            if (this.layout.squareThumbs) {
                return this.getHeight;
            }
            else {
                if (!this.vertical) {
                    return Math.ceil(file.imageWidth / (file.imageHeight / (this.layout.elements*20))) + 'px';
                }
                else {
                    return Math.ceil(file.imageHeight / (file.imageWidth / (this.layout.elements*20))) + 'px';
                }
            }
        },

        onMouseDown(e)
        {
            this.isPanning = true;
            this.panPos = e.pageX - this.container[0].offsetLeft;
            this.panOrg = this.container[0].scrollLeft;
        },
        onMouseUp(e)
        {
            this.isPanning = false;
        },
        onMouseLeave(e)
        {
            this.isPanning = false;
        },
        onMouseMove(e)
        {
            if (this.isPanning) {
                e.preventDefault();
                const x = e.pageX - this.container[0].offsetLeft;
                const delta = (x - this.panPos) * 2;  // pan speed
                this.container[0].scrollLeft = this.panOrg - delta;
            }
        },

        itemClickedLocal(id,event)
        {
            if (this.isPanning) {
                this.isPanning = false;
                if (Math.abs(this.panOrg,this.container[0].scrollLeft) < 5) {
                    // Not panned
                    this.itemClicked(id,event);
                }
            }
            else {
                // Call the base class directly.
                this.itemClicked(id,event);
            }
        },

        handleWheel: function(e) {
            let divider = 1;
            this.container.animate({scrollLeft: `+=${e.deltaY/divider}`},50);
        },

        goto(where)
        {
            let curFocus = this.layout.focusedFile;
            let ficur = this._getFocusIndex();
            
            // Call the navigate method in the base component.
            // Vue extends is not a real ES6 derivation chain so there is no super.
            this.navigate(where);

            if (curFocus != this.layout.focusedFile) {
                this.onFocusChanged();
            }
        },

        onFocusChanged()
        {
            let fi = this._getFocusIndex();
            if (fi >= 0) {
                // Get the element for the new focused file and bring it into 
                // view
                let element = $(`#${this.getFileId(this.group.files[fi])}`)[0];
                let elementWidth = $(element).width();
                let pos = element.offsetLeft;

                if (pos-(elementWidth/2) <= this.container.scrollLeft()) {
                    // Scroll left
                    this.container.animate({scrollLeft: pos - elementWidth},50);
                }
                else if (pos+elementWidth+(elementWidth/2) >= this.container.scrollLeft()+this.container.width()) {
                    // Scroll right
                    this.container.animate({scrollLeft: pos - this.container.width() + elementWidth},50);
                }
            }
        }
    },

    template: `
        <div :id="'fw-strip-'+group.id" class="d-flex flex-row fw-strip-container"
        
        tabindex="0"
        @keydown.left.stop="goto('p')"
        @keydown.right.stop="goto('n')"
        @keydown.home.stop="goto('f')"
        @keydown.end.stop="goto('l')"
        @wheel="handleWheel($event)"
        @mousedown="onMouseDown($event)" 
        @mouseup="onMouseUp($event)" 
        @mouseleave="onMouseLeave($event)" 
        @mousemove="onMouseMove($event)"
        >
            <div class="flex-grow-1" style="width: 1px;">
                <div class="d-flex" :class="{'flex-row' : !vertical, 'flex-column' : vertical}" :style="{'min-height':getHeight}" @click="itemClickedLocal(0,$event)">
                    <div v-if="group.files" v-for="f,index in group.files" :id="getFileId(f)" class="fw-strip-element" @click.stop="itemClickedLocal(f.id,$event)">
                        <div class="card fw-strip-card" :class="getSelectionClass(f.id)" :style="{'height': vertical ? getDynamicSize(f) : getHeight,'width': vertical ? getHeight : getDynamicSize(f)}">

                            <div class="card-body fw-strip-card-header" v-if="getHeader(f)" v-html="getHeader(f)"></div>

                            <div v-if="layout.squareThumbs" :data-id="f.id" :data-src="getImageUrl(f)" class="ptc-fw-observed-image" :style="{'background-image': 'url(' + getPlaceholderUrl(f) + ')','width':getHeight, 'height': getHeight, 'background-repeat': 'no-repeat', 'background-size': 'cover', 'background-position':'center center'}"></div>
                            <img v-if="!layout.squareThumbs" :data-id="f.id" :data-src="getImageUrl(f)" :src="getPlaceholderUrl(f)" :alt="f.name" class="card-img-top fw-strip-image ptc-fw-observed-image" :style="{'width':getWidth(f),'height': getHeight}">

                            <ptc-fw-cols
                                v-if="layout.collectionOptions.enabled"
                                :file="f"
                                :options="layout.collectionOptions"
                            ></ptc-fw-cols>

                            <div class="card-body fw-strip-card-footer" v-if="getFooter(f)" v-html="getFooter(f)"></div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    `
}


